import {
  Button,
  Col,
  Divider,
  Modal,
  Progress,
  Row,
  Select,
  Space,
  message,
} from "antd";
import { useDicomUploader } from "./DicomUploader";
import { Series, Study } from "./DicomUploader/util";
import { DicomTable } from "./DicomTable";
import { useDomain } from "@/hook/useDomain";
import { Typography } from "antd";
import { useState } from "react";
import { User } from "@@/domain/User/entities/User";
import { limitConcurrency } from "./limitConcurrency";
import { CloudUploadOutlined, InboxOutlined } from "@ant-design/icons";
import { openOHIFViewer, flatternStudies, FlatStudyItem } from "./util";

const { Text } = Typography;

interface DicomUploadProps {
  children?: JSX.Element;
}

export const DicomUpload = (props: DicomUploadProps) => {
  const [messageApi, contextHolder] = message.useMessage();
  const { UploadInput, fileCalculator, studys, isLoading } = useDicomUploader();
  const { dicomDomainService, userDomainService } = useDomain();
  const { dcmFileNum, totalFileNum, dcmFileSize } = fileCalculator;
  const [selectRows, setSelectedRows] = useState<Study[]>([]);
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [annotators, setAnnotators] = useState<User[]>([]);
  const [selectAnnotator, setSelectAnnotator] = useState<User | undefined>(
    undefined
  );
  const [visible, setVisible] = useState<boolean>(false);
  const [percent, setPercent] = useState<number>(0);

  /**
   * 单个序列阅片
   */
  const onUploadFiles = async (study: Study, series: Series) => {
    const { SeriesInstanceUID, subs } = series;
    const { StudyInstanceUID } = study;
    const instances = await dicomDomainService.existInPacs({
      SeriesInstanceUID,
      StudyInstanceUID,
    });
    // pacs已存在
    if (instances.length === subs.length) {
      openOHIFViewer(StudyInstanceUID, SeriesInstanceUID);
    } else {
      setVisible(true);
      limitConcurrency(
        series.subs.map((f: File) => () => dicomDomainService.upload2Pacs(f)),
        10,
        (completed, total) => {
          console.log(`${completed} out of ${total} tasks completed.`);
          setPercent(Math.floor((completed / total) * 100));
        },
        () => {
          setVisible(false);
          setPercent(0);
          openOHIFViewer(StudyInstanceUID, SeriesInstanceUID);
        }
      );
    }
  };

  /**
   * 上传全部
   */
  const onUploadEntiredDicom = () => {
    const items = flatternStudies(studys);
    // 过滤存在pacs中的序列
    const checkExistPromises = items.map(
      (item) =>
        new Promise(async (resolve) => {
          const instances = await dicomDomainService.existInPacs({
            SeriesInstanceUID: item.SeriesInstanceUID,
            StudyInstanceUID: item.StudyInstanceUID,
          });
          const existInPacs = instances.length === item.Files.length;
          resolve(existInPacs ? false : item);
        })
    ) as Promise<FlatStudyItem | false>[];
    // 上传到pacs
    Promise.all(checkExistPromises).then((result) => {
      const files = result.map((i) => (!i ? [] : i.Files)).flat();
      if (files.length === 0) return messageApi.info("全部影像均已存在pacs中");
      setVisible(true);
      limitConcurrency(
        files.map((f: File) => () => dicomDomainService.upload2Pacs(f)),
        10,
        (completed, total) => {
          console.log(`${completed} out of ${total} tasks completed.`);
          setPercent(Math.floor((completed / total) * 100));
        },
        () => {
          setVisible(false);
          setPercent(0);
        }
      );
    });
  };

  const onClickAssign = () => {
    userDomainService.getDmpAnnotators().then((res) => {
      setIsModalOpen(true);
      setAnnotators(res.data as User[]);
    });
  };

  /**
   * 统计信息
   */
  const DicomFileInfo = () =>
    !!totalFileNum && (
      <Space>
        <Text type="secondary">扫描总文件数: {totalFileNum}</Text>
        <Divider type="vertical" />
        <Text type="secondary">包含: {dcmFileNum} 个dicom文件</Text>
        <Divider type="vertical" />
        <Text type="secondary">dcm文件总体积: {dcmFileSize} </Text>
      </Space>
    );

  /**
   * 分配任务
   */
  const onAssignConfirm = () => {
    if (!selectAnnotator?.id) return;
    userDomainService.assignLabelDicom(selectAnnotator, selectRows);
    setSelectAnnotator(undefined);
    setIsModalOpen(false);
  };

  return (
    <div style={{ padding: 20 }}>
      <Row style={{ paddingBottom: 20 }}>
        <Col span={24}>
          <Space>
            <UploadInput />
            <Button
              icon={<CloudUploadOutlined />}
              disabled={Number(dcmFileNum) === 0}
              type="primary"
              onClick={onUploadEntiredDicom}
            >
              上传全部
            </Button>
            <Button
              icon={<InboxOutlined />}
              disabled={selectRows.length === 0}
              type="primary"
              onClick={onClickAssign}
            >
              分配
            </Button>
          </Space>
        </Col>
      </Row>
      <DicomFileInfo />
      <DicomTable
        studys={studys}
        loading={isLoading}
        onSelectedRows={(rows) => setSelectedRows(rows)}
        onUploadFiles={onUploadFiles}
      />
      <Modal
        width={320}
        title="选择标注"
        open={isModalOpen}
        closable
        cancelText="再想想"
        okText="确定"
        onOk={onAssignConfirm}
        onCancel={() => setIsModalOpen(false)}
      >
        <Select
          style={{ width: "100%", marginBottom: 20 }}
          placeholder="选择标注"
          value={selectAnnotator?.id as number}
          onChange={(id: number) =>
            setSelectAnnotator(annotators.find((a) => a.id === id))
          }
          options={annotators.map((a) => ({
            value: a.id,
            label: a.username,
          }))}
        />
      </Modal>
      <Modal title="上传影像中" open={visible} footer={null} closable={false}>
        <Progress percent={percent} strokeLinecap="butt" />
      </Modal>
      {contextHolder}
    </div>
  );
};
